@@ -1,11 +1,18 @@
 import copy
-from lark import Transformer, v_args, Token
-from optimade.server.mappers import BaseResourceMapper
+from lark import v_args, Token
+from optimade.filtertransformers.base_transformer import BaseTransformer
 from optimade.server.exceptions import BadRequest
 
+__all__ = ("MongoTransformer",)
 
-class MongoTransformer(Transformer):
-    """Support for grammar v0.10.1"""
+
+class MongoTransformer(BaseTransformer):
+    """Transformer for MongoDB backend. Parses lark tree into
+    a dictionary to be passed to pymongo/mongomock. Uses
+    post-processing functions to handle aliasing and some
+    specific edge-cases for MongoDB.
+
+    """
 
     operator_map = {
         "<": "$lt",
@@ -24,14 +31,6 @@ class MongoTransformer(Transformer):
         "$eq": "$eq",
     }
 
-    def __init__(self, mapper: BaseResourceMapper = None):
-        """Initialise the object, optionally loading in a
-        resource mapper for use when post-processing.
-
-        """
-        self.mapper = mapper
-        super().__init__()
-
     def postprocess(self, query):
         """ Used to post-process the final parsed query. """
         if self.mapper:
@@ -46,41 +45,6 @@ class MongoTransformer(Transformer):
 
         return query
 
-    def transform(self, tree):
-        return self.postprocess(super().transform(tree))
-
-    def filter(self, arg):
-        # filter: expression*
-        return arg[0] if arg else None
-
-    @v_args(inline=True)
-    def constant(self, value):
-        # constant: string | number
-        # Note: Do nothing!
-        return value
-
-    @v_args(inline=True)
-    def value(self, value):
-        # value: string | number | property
-        # Note: Do nothing!
-        return value
-
-    @v_args(inline=True)
-    def non_string_value(self, value):
-        """ non_string_value: number | property """
-        # Note: Do nothing!
-        return value
-
-    @v_args(inline=True)
-    def not_implemented_string(self, value):
-        """not_implemented_string: value
-
-        Raise NotImplementedError.
-        For further information, see Materials-Consortia/OPTIMADE issue 157:
-        https://github.com/Materials-Consortia/OPTIMADE/issues/157
-        """
-        raise NotImplementedError("Comparing strings is not yet implemented.")
-
     def value_list(self, arg):
         # value_list: [ OPERATOR ] value ( "," [ OPERATOR ] value )*
         # NOTE: no support for optional OPERATOR, yet, so this takes the
@@ -115,12 +79,6 @@ class MongoTransformer(Transformer):
         # expression_phrase: [ NOT ] ( comparison | "(" expression ")" )
         return self._recursive_expression_phrase(arg)
 
-    @v_args(inline=True)
-    def comparison(self, value):
-        # comparison: constant_first_comparison | property_first_comparison
-        # Note: Do nothing!
-        return value
-
     def property_first_comparison(self, arg):
         # property_first_comparison: property ( value_op_rhs | known_op_rhs | fuzzy_string_op_rhs | set_op_rhs |
         # set_zip_op_rhs | length_op_rhs )
@@ -204,34 +162,6 @@ class MongoTransformer(Transformer):
         # property_zip_addon: ":" property (":" property)*
         raise NotImplementedError
 
-    def property(self, arg):
-        # property: IDENTIFIER ( "." IDENTIFIER )*
-        return ".".join(arg)
-
-    @v_args(inline=True)
-    def string(self, string):
-        # string: ESCAPED_STRING
-        return string.strip('"')
-
-    @v_args(inline=True)
-    def signed_int(self, number):
-        # signed_int : SIGNED_INT
-        return int(number)
-
-    @v_args(inline=True)
-    def number(self, number):
-        # number: SIGNED_INT | SIGNED_FLOAT
-        if number.type == "SIGNED_INT":
-            type_ = int
-        elif number.type == "SIGNED_FLOAT":
-            type_ = float
-        return type_(number)
-
-    def __default__(self, data, children, meta):
-        raise NotImplementedError(
-            f"Calling __default__, i.e., unknown grammar concept. data: {data}, children: {children}, meta: {meta}"
-        )
-
     def _recursive_expression_phrase(self, arg):
         """Helper function for parsing `expression_phrase`. Recursively sorts out
         the correct precedence for `$and`, `$or` and `$nor`.
