@@ -1,7 +1,7 @@
 from fastapi import Request, APIRouter
 from fastapi.responses import Response
 
-from .utils import BASE_URL_PREFIXES
+from optimade.server.routers.utils import BASE_URL_PREFIXES
 
 router = APIRouter(redirect_slashes=True)
 
@@ -11,7 +11,9 @@ class CsvResponse(Response):
 
 
 @router.get(
-    "/versions", tags=["Versions"], response_class=CsvResponse,
+    "/versions",
+    tags=["Versions"],
+    response_class=CsvResponse,
 )
 def get_versions(request: Request):
     """Respond with the text/csv representation for the served versions."""
